# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-

# Copyright (C) 2014 nfschina.
# Written by CDOS Support Team <support@cpu-os.ac.cn>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import print_function

import os
import json

import syslog

from gi.repository import Gtk

from ubiquity import plugin, misc

NAME = 'addtionalpkgs'
# AFTER = ['usersetup', 'protocol']
HIDDEN = ['addtionalpkgs']
WEIGHT = 10


class PageGtk(plugin.PluginUI):

    plugin_title = 'ubiquity/text/additionalpkgs_heading_label'
    help_title = 'ubiquity/help/title'
    help_filename = "additional_pkgs"

    def __init__(self, controller, *args, **kwargs):
        self.controller = controller
        self.config = {}
        self.startrow = 0
        self.thirdpart_pkgs = {}

        builder = Gtk.Builder()
        self.controller.add_builder(builder)
        builder.add_from_file(os.path.join(
            os.environ['UBIQUITY_GLADE'], 'stepPreInstallApp.ui'))
        builder.connect_signals(self)

        self.option_grid_box = builder.get_object("option_grid_box")
        self.option_app_window = builder.get_object("option_app_window")
        self.main_stack_view = builder.get_object("main_stack_view")
        self.thirdpart_tree_model = builder.get_object("thirdpart_tree_model")
        self.thirdpart_app_window = builder.get_object("thirdpart_app_window")
        self.pkgname_column = builder.get_object("pkgname_column")
        self.old_ver_column = builder.get_object("old_ver_column")
        self.new_ver_column = builder.get_object("new_ver_column")
        self.stack_controller = builder.get_object("stack_controller")

        self.page = builder.get_object('stepPreInstallApp')

        self.plugin_widgets = self.page

    def on_thirdpart_pkg_toggled(self, cell, path_str):
        return

    def load_pkglist(self, jsonpath):
        self.load_pkglist_from_json(jsonpath)

    def load_pkglist_from_json(self, jsonpath):
        jsonobj = None
        try:
            with open(jsonpath, "r") as fd:
                jsonstring = fd.read()
                jsonobj = json.loads(jsonstring)
        except Exception as e:
            syslog.syslog(syslog.LOG_WARNING,
                "ubi-additionalpkgs -----> load_pkglist_from_json failed: " + e)
            return -1

        startcol = 0
        keylist = list(jsonobj.keys())
        keylist.sort()
        for key in keylist:
            info = jsonobj[key]
            if info is None or "pkglist" not in info:
                continue

            checkOptions = self.createCheckOptions(info, key)

            self.option_grid_box.insert_row(self.startrow)
            self.option_grid_box.attach(checkOptions, startcol + 1, self.startrow, 1, 1)
            self.startrow += 1

        if self.main_stack_view.get_child_by_name("option_app") is None:
            self.main_stack_view.add_titled(self.option_app_window,
                'option_app', self.controller.get_string('ubiquity/text/plugin_addpkgs_option_app'))

        return 0

    def createCheckOptions(self, info, key):
        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        vbox.set_margin_left(20)
        pkgTypeLabel = self.createTypeLabel(info, key)
        hbox1 = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        hbox1.set_margin_left(0)
        hbox1.set_margin_top(20)
        hbox1.pack_start(pkgTypeLabel, False, False, 0)
        vbox.pack_start(hbox1, False, False, 0)
        pkglist = info["pkglist"]
        for item in pkglist:
            hbox2 = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
            hbox2.set_margin_left(30)
            hbox2.set_margin_top(20)
            check = Gtk.CheckButton.new_with_label(item["mfrs"])
            check.set_property("can_focus", False)
            setattr(check, "pkginfo", item)
            if item["active"]:
                check.set_active(True)
            self.allCheckbutton.append(check)
            hbox2.pack_start(check, False, False, 0)
            vbox.pack_start(hbox2, False, False, 0)
        vbox.show_all()
        return vbox

    def createTypeLabel(self, pkgInfo, defaultTypeName):
        typename = defaultTypeName
        if "trans-zh_CN" in pkgInfo:
            typename = pkgInfo["trans-zh_CN"]
        elif "trans" in pkgInfo:
            typename = pkgInfo["trans"]

        pkgTypeLabel = Gtk.Label.new(typename)
        pkgTypeLabel.set_margin_top(20)
        pkgTypeLabel.show()

        return pkgTypeLabel

    def set_config_json(self, path):
        self.allCheckbutton = []
        while self.startrow > 0:
            self.option_grid_box.remove_row(0)
            self.startrow -= 1

        self.startrow = 0

        loadPkgResult = self.load_pkglist(path)
        if loadPkgResult == -1:
            print("Json file error.")
            self.controller.go_forward()

    def plugin_on_help_clicked(self):
        self.show_help(self.controller.get_string(self.help_title), self.help_filename)

    @misc.raise_privileges
    def plugin_on_next_clicked(self):
        needInstallPkginfoList = []
        needInstallPkgsList = []
        try:
            pkgpath = '/tmp/nfs_3party_pkgs_need_install'
            if os.path.exists(pkgpath):
                misc.execute('rm', '-f', pkgpath)

            for button in self.allCheckbutton:
                if button.get_active():
                    info = getattr(button, "pkginfo")
                    if info is not None and "name" in info:
                        needInstallPkginfoList.append(info)

            needInstallPkginfoList = sorted(needInstallPkginfoList, key=lambda x: x['installorder'])

            for info in needInstallPkginfoList:
                name = info['name']
                name.strip()
                if len(name) > 0:
                    needInstallPkgsList.append(name)

            if len(needInstallPkgsList) > 0:
                installpkginfo = {}
                installpkginfo["pkglist"] = needInstallPkgsList
                with open(pkgpath, "w") as fd:
                    fd.write(json.dumps(installpkginfo))
        finally:
            return False

    def plugin_translate(self, lang):
        self.pkgname_column.set_title(self.controller.get_string("ubiquity/text/plugin_addpkgs_thirdpart_coltitle_pkg"))
        self.old_ver_column.set_title(self.controller.get_string("ubiquity/text/plugin_addpkgs_thirdpart_coltitle_oldver"))
        self.new_ver_column.set_title(self.controller.get_string("ubiquity/text/plugin_addpkgs_thirdpart_coltitle_newver"))


class Page(plugin.Plugin):
    def prepare(self, unfiltered=False):
        fullCommand = ['sh', '-c']
        commandstr = ""
        fullQuestion = []
        fullenviron = {}

        commandstr += '/usr/share/ubiquity/load_extra_pkgs.sh'
        fullQuestion += ['ubiquity/extra_pkgs/config_json']

        fullCommand.append(commandstr)

        return fullCommand, fullQuestion, fullenviron

    def run(self, priority, question):
        if question == 'ubiquity/extra_pkgs/config_json':
            configjson = self.db.get(question)
            self.ui.set_config_json(configjson)
            return plugin.Plugin.run(self, priority, question)
        return plugin.Plugin.run(self, priority, question)
