# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-

# Copyright (C) 2014 nfschina.
# Written by Yuanzhe <yuanzhe@iscas.ac.cn>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import print_function

import os

from ubiquity import plugin


NAME = 'alternative'
AFTER = 'usersetup'
WEIGHT = 11
OEM = False


# TODO: This cannot be a non-debconf plugin after all as OEMs may want to
# preseed the 'install updates' and 'install non-free software' options. So?
# Just db_get them. No need for any other overhead, surely. Actually, you
# need the dbfilter for that get.


class PageGtk(plugin.PluginUI):

    plugin_title = 'ubiquity/text/alternative_heading_label'
    help_dialog = 'help_alternative'
    help_label = 'alternative_text_label'
    help_prompt = 'prompt_alternative'

    def __init__(self, controller, *args, **kwargs):
        self.controller = controller
        from gi.repository import Gtk
        builder = Gtk.Builder()
        self.controller.add_builder(builder)
        builder.add_from_file(os.path.join(
            os.environ['UBIQUITY_GLADE'], 'stepAlternative.ui'))
        builder.connect_signals(self)

        self.kingsoft = builder.get_object('kingsoft-office')
        haswps = os.system("dpkg -l | grep wps-office > /dev/null")
        if (haswps != 0):
            self.kingsoft.hide()

        self.page = builder.get_object('stepAlternative')
        self.select_all = builder.get_object('select_all')
        self.alternative_vbox = builder.get_object('alternative_vbox')
        self.plugin_widgets = self.page

    def on_select_all_clicked(self, widget):
        checkbuttons = self.alternative_vbox.get_children()
        if self.select_all.get_active():
            for checkbutton in checkbuttons:
                checkbutton.set_active(True)
        else:
            for checkbutton in checkbuttons:
                checkbutton.set_active(False)
        self.select_all.set_inconsistent(False)

    def on_select_child_clicked(self, widget):
        if not widget.get_active():
            self.select_all.set_inconsistent(True)

    def plugin_on_help_clicked(self):
        self.show_help('/usr/share/ubiquity/gtk/stepHelp.ui', self.help_dialog, self.help_label, self.help_prompt)


class Page(plugin.Plugin):
    def prepare(self):
        command = ['/usr/share/ubiquity/simple-plugins', 'alternative']
        questions = ['ubiquity/user_alternative']
        return command, questions

    def ok_handler(self):
        user_alternative = []
        checkbuttons = self.ui.alternative_vbox.get_children()
        for checkbutton in checkbuttons:
            if not checkbutton.get_active():
                user_alternative.append(checkbutton.get_name())
        self.preseed('ubiquity/user_alternative', ' '.join(user_alternative))
        plugin.Plugin.ok_handler(self)
