# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2009 Canonical Ltd.
# Written by Michael Terry <michael.terry@canonical.com>.
#
# This file is part of Ubiquity.
#
# Ubiquity is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Ubiquity is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ubiquity.  If not, see <http://www.gnu.org/licenses/>.

from ubiquity.filteredcommand import FilteredCommand, UntrustedBase
from gi.repository import Gtk
import os


class PluginUI(UntrustedBase):
    # We define an init even if empty so that arguments that we give but are
    # not used don't cause an error.
    def __init__(self, *args, **kwargs):
        pass

    def show_help(self, title, helpfilename):
        self.builder = Gtk.Builder()
        self.builder.add_from_file('/usr/share/ubiquity/gtk/stepHelp.ui')
        self.builder.connect_signals(self)
        self.window = self.builder.get_object("help_window")
        self.label_info = self.builder.get_object("help_main_contents")
        self.label_prompt = self.builder.get_object("help_prompt")
        self.button_helpclose = self.builder.get_object("closebutton1")
        fullFileName = "/usr/share/ubiquity/helpFiles"
        try:
            if self.controller._wizard.locale.startswith('zh_CN'):
                fullFileName += "/zh_CN/%s.txt" % (helpfilename)
                self.close_text = '关闭(C)'
            else:
                fullFileName += "/en/%s.txt" % (helpfilename)
                self.close_text = 'Close'

            if os.path.isfile(fullFileName):
                self.text = open(fullFileName).read()
        except:
            self.text = "No Help file found"
        self.label_info.set_markup(self.text)
        self.button_helpclose.set_label(self.close_text)
        self.window.set_title(title)
        self.window.connect("delete-event", self.on_help_cross_clicked)
        self.window.run()

    def find_change(self, widget):
        self.entry_text = widget.get_text()
        self.entry_text_posi = self.text.upper().find(self.entry_text.upper())
        self.label_info.set_markup(self.text)
        if self.entry_text_posi != -1:
            self.label_prompt.set_markup("")
            self.entry_text = self.text[self.entry_text_posi:len(self.entry_text) + self.entry_text_posi]
            self.changetext = self.text.replace(self.entry_text, "<span color=\"#007dce\">" + self.entry_text + "</span>")
            self.label_info.set_markup(self.changetext)
        else:
            self.label_prompt.set_markup("<small><span color=\"darkred\">未找到短语</span></small>")

    def closs_help(self, argc):
        self.window.destroy()

    def on_help_cross_clicked(self, argc, no_meaning):
        self.window.destroy()
        return True


class Plugin(FilteredCommand):
    def prepare(self, unfiltered=False):
        # None causes dbfilter to just spin a main loop and wait for OK/Cancel
        return None


class InstallPlugin(Plugin):
    def install(self, *args, **kwargs):
        return self.run_command(auto_process=True)


# Use this as a decorator if you want to guard against a function being called
# when on a different page.  For instance, when it is called as part of the
# event loop.
def only_this_page(target):
    def wrapper(self, *args, **kwargs):
        if self.controller.dbfilter:
            return target(self, *args, **kwargs)
        else:
            # gobject removes timeouts if they return non-True.
            return True
    return wrapper
